@@ -1,3 +1,6 @@
+import logging
+
+
 class SeqSplitConfig:
     """Configuration class for sequence splitting behavior.
 
@@ -105,6 +108,8 @@ class AncComposerBuffer:
             int: Token length of the item, default to 1 if get_token_length_fn is not provided
         """
         if self.get_token_length_fn is not None:
+            if item is None:
+                return 0
             return self.get_token_length_fn(item)
         else:
             return 1
@@ -362,19 +367,38 @@ class AncComposer:
         Returns:
             list: List of batched items or None if no batches were created
         """
-        total_token_length = self.buffer.put(sample)
+        if (
+            self.allow_split
+            or sample is None
+            or self.buffer._get_token_length(sample) <= self.max_seq_len
+        ):
+            total_token_length = self.buffer.put(sample)
+        else:
+            # don't put this sample into buffer, it will stay there forever
+            logging.warning(
+                f"sample token length {self.buffer._get_token_length(sample)} exceeded max_seq_len, skipping"
+            )
+            total_token_length = self.buffer.put(None)
+
         res = []
         while total_token_length >= self.max_seq_len * self.ratio:
             items = self.get_items()
             res.append(items)
-            total_token_length = self.buffer.put(None)
+            new_total = self.buffer.put(None)
+            if new_total >= total_token_length:
+                # avoid infinite loop when no item can be fetched from buffer
+                break
+            total_token_length = new_total
         if is_last_call:
             self.buffer.combine_buffers()
             total_token_length = self.buffer.put(None)
             while total_token_length >= self.max_seq_len:
                 items = self.get_items(ignore_split_freezing=True)
                 res.append(items)
-                total_token_length = self.buffer.put(None)
+                new_total = self.buffer.put(None)
+                if new_total >= total_token_length:
+                    break
+                total_token_length = new_total
             if total_token_length > 0:
                 items = self.get_items(is_last_call=True)
                 res.append(items)
