@@ -10,7 +10,7 @@ from anc_data.anc_processor import Processor
 from anc_data.anc_sampler import AncSampler
 from anc_data.jsonl_dataset import JsonlDataset
 from anc_data.parquet_dataset import ParquetConcateDataset
-from anc_data.utils import DataNotReady
+from anc_data.utils import DataNotReady, EndOfIteration
 
 
 def assert_2_layer_list(x):
@@ -79,6 +79,7 @@ class AncDataset(IterableDataset):
         self.processor = processor
         self.sampler = sampler
         self.enable_compose = enable_compose
+        self.processor.enable_compose = enable_compose
         self.composer = None
         if self.enable_compose:
             assert hasattr(processor, "get_token_length_fn")
@@ -124,39 +125,33 @@ class AncDataset(IterableDataset):
 
     def get_generator(self, sampler_iter, wid, num_workers):
         while True:
-            try:
-                indices = next(sampler_iter)
-            except StopIteration:
+            indices = next(sampler_iter, EndOfIteration)
+            if indices is EndOfIteration:
                 logging.warning(
-                    f"worker {wid} of {num_workers} get StopIteration from sampler, return DataNotReady"
+                    f"worker {wid} of {num_workers} get EndOfIteration from sampler, return DataNotReady"
                 )
                 yield (wid, DataNotReady(), True)
                 break
             item_generator = self._getitem(indices)
             is_last = False
             while True:
-                try:
-                    # Note: is_last isn't set if item_generator returns nothing due to StopIteration
-                    next_data, is_last = next(item_generator)
-                    if isinstance(next_data, DataNotReady):
-                        # yield (wid, next_data, False)
-                        continue
-                    else:
-                        yield (wid, [next_data], False)
-                        self.cur_step += 1
-                        if (
-                            self.ckpt_interval is not None
-                            and self.cur_step % self.ckpt_interval == 0
-                        ):
-                            if self.state_queues is not None:
-                                self.state_queues[wid].put(self._get_ckpt())
-                # RuntimeError should be a wrapper of StopIteration due to PEP 479
-                except (StopIteration, RuntimeError):
-                    # TODO: check if this is correct when next() throws
-                    # is_last = indices[-1] # True
+                result = next(item_generator, EndOfIteration)
+                if result is EndOfIteration:
                     if is_last:
                         yield (wid, DataNotReady(), True)
                     break
+                next_data, is_last = result
+                if isinstance(next_data, DataNotReady):
+                    continue
+
+                yield (wid, [next_data], False)
+                self.cur_step += 1
+                if (
+                    self.ckpt_interval is not None
+                    and self.cur_step % self.ckpt_interval == 0
+                ):
+                    if self.state_queues is not None:
+                        self.state_queues[wid].put(self._get_ckpt())
             if is_last:
                 break
 
