@@ -1,3 +1,4 @@
+import bisect
 import copy
 import logging
 import math
@@ -6,6 +7,8 @@ import random
 import numpy as np
 from torch.utils.data.distributed import DistributedSampler
 
+from anc_data.utils import EndOfIteration
+
 
 class CustomizedRange:
     def __init__(self, start, end, seed=0):
@@ -49,36 +52,48 @@ class CustomizedRange:
 class RangesHolder:
     def __init__(self, list_of_ranges):
         self.list_of_ranges = list_of_ranges
+        self.accu_lengths = self.create_accu_length(list_of_ranges)
         self.length = sum(len(r) for r in list_of_ranges)
         self.expanded = {}
+        self.i = 0
 
     def __len__(self):
         return self.length
 
+    def create_accu_length(self, list_of_ranges):
+        accu_length = 0
+        accu_lengths = []
+        for rng in list_of_ranges:
+            accu_lengths.append(accu_length)
+            accu_length += len(rng)
+        accu_lengths.append(accu_length)
+        return accu_lengths
+
+    def _get_start_idx(self, start):
+        idx = bisect.bisect_right(self.accu_lengths, start) - 1
+        assert 0 <= idx < len(self.accu_lengths)
+        return idx
+
     def get_start_and_end_ranges(self, start, length):
         start_range_id, start_range_offset = 0, 0
-        accu_length = 0
+        n = len(self.list_of_ranges)
         end = start + length
-        end_range_id = len(self.list_of_ranges) - 1
+        end_range_id = n - 1
         end_range_offset = len(self.list_of_ranges[end_range_id])
-        for i in range(len(self.list_of_ranges)):
-            if start < accu_length + len(self.list_of_ranges[i]):
-                start_range_id = i
-                start_range_offset = start - accu_length
-                break
-            accu_length += len(self.list_of_ranges[i])
-        for i in range(start_range_id, len(self.list_of_ranges)):
+        if self.accu_lengths[self.i] <= start < self.accu_lengths[self.i + 1]:
+            start_range_id = self.i
+        else:
+            start_range_id = self._get_start_idx(start)
+        start_range_offset = start - self.accu_lengths[start_range_id]
+        accu_length = self.accu_lengths[start_range_id]
+        for i in range(start_range_id, n):
             accu_length += len(self.list_of_ranges[i])
             if end < accu_length:
                 end_range_id = i
                 end_range_offset = end - (accu_length - len(self.list_of_ranges[i]))
                 break
-        return (
-            start_range_id,
-            start_range_offset,
-            end_range_id,
-            end_range_offset,
-        )
+        self.i = end_range_id
+        return start_range_id, start_range_offset, end_range_id, end_range_offset
 
     def slice(self, start, length):
         start_range_id, start_range_offset, end_range_id, end_range_offset = (
@@ -159,6 +174,7 @@ class RangesHolder:
     def random_reset(self, idx, offset=1):
         self.expanded = {}
         self.list_of_ranges = self.list_of_ranges[idx:] + self.list_of_ranges[:idx]
+        self.accu_lengths = self.create_accu_length(self.list_of_ranges)
         for range in self.list_of_ranges:
             # reset each range's seed by moving it forward by offset
             range._seed += offset
@@ -221,7 +237,6 @@ class AncSampler(DistributedSampler):
         self.resume_step = resume_step
         self.step = resume_step
         self.seed = seed
-        self._random = random.Random(seed)
         self._np_random = np.random.default_rng(seed)
         self.last_iter_epoch = -1
         self.indices = None
@@ -304,7 +319,7 @@ class AncSampler(DistributedSampler):
                 start = (
                     0
                     if not self.shuffle
-                    else self._random.randint(0, len(tmp_indices) - length)
+                    else self._np_random.integers(0, len(tmp_indices) - length)
                 )
                 res.append(tmp_indices.slice(start, length))
             else:
@@ -327,7 +342,7 @@ class AncSampler(DistributedSampler):
                 else:
                     res.append(tmp_indices)
         if self.shuffle:
-            self._random.shuffle(res)
+            self._np_random.shuffle(res)
         return res
 
     def _get_indices(self, sub_lengths, offset=0, return_sub_indices=False, ratio=[1]):
@@ -353,7 +368,7 @@ class AncSampler(DistributedSampler):
             accumulate_length += item
 
         if self.shuffle:
-            self._random.shuffle(indices_list)
+            self._np_random.shuffle(indices_list)
         if return_sub_indices:
             return indices_list
         if not indices_list:
@@ -390,7 +405,6 @@ class AncSampler(DistributedSampler):
             )
 
         if self.repeat and self.shuffle:
-            self._random.seed(self.seed)
             self._np_random = np.random.default_rng(self.seed)
         while True:
             self.inner_epoch_count += 1
@@ -425,10 +439,12 @@ class AncSampler(DistributedSampler):
                 self._np_random.shuffle(indices)
             elif self.shuffle:
                 if self.mem_save_mode:
-                    shift_offset = self._random.randint(0, len(indices.list_of_ranges))
+                    shift_offset = self._np_random.integers(
+                        0, len(indices.list_of_ranges)
+                    )
                     indices.random_reset(shift_offset)
                 else:
-                    shift_offset = self._random.randint(0, len(indices))
+                    shift_offset = self._np_random.integers(0, len(indices))
                     indices = np.concatenate(
                         (indices[shift_offset:], indices[:shift_offset])
                     )
@@ -449,7 +465,7 @@ class AncSampler(DistributedSampler):
                     accumulate_length += sum(item)
                     sub_indices_list += sub_indices
                 if self.shuffle:
-                    self._random.shuffle(sub_indices_list)
+                    self._np_random.shuffle(sub_indices_list)
                 if not self.mem_save_mode:
                     if not sub_indices_list:
                         indices = np.array([], dtype=np.uint32)
@@ -513,11 +529,11 @@ class AncSampler(DistributedSampler):
             yield [], False
 
         while True:
-            try:
-                self.step += 1
-                yield next(indices_generator)
-            except StopIteration:
+            self.step += 1
+            indices = next(indices_generator, EndOfIteration)
+            if indices is EndOfIteration:
                 break
+            yield indices
 
     def set_step(self, step):
         self.resume_step = step
@@ -535,6 +551,8 @@ class AncSampler(DistributedSampler):
 
     def _set_ckpt(self, state):
         self.__dict__.update(state)
+        # reset random generators, their states will be resumed by resume_step
+        self._np_random = np.random.default_rng(self.seed)
 
     def _get_ckpt(self):
         indices = self.__dict__.pop("indices", None)
@@ -542,6 +560,7 @@ class AncSampler(DistributedSampler):
         ckpt_state["resume_step"] = self.step
         ckpt_state["last_iter_epoch"] = -1
         self.__dict__["indices"] = indices
+        del ckpt_state["_np_random"]
         return ckpt_state
 
 
