@@ -1,4 +1,5 @@
 import copy
+import functools
 import multiprocessing.dummy as mt
 import os
 
@@ -53,10 +54,10 @@ def process_filepath_columns(filepath, columns, file_system=None):
     assert isinstance(
         filepath, list
     ), f"filepath should be a list but got {type(filepath)}"
-    assert len(filepath) > 0
+    assert len(filepath) > 0, "filepath should be a non-empty list"
     # if filepath is a list of [meta, wav, ...], the first one is the main one and must exist
     # if filepath only contains one file, it must exist
-    assert path_exists(filepath[0], file_system)
+    assert path_exists(filepath[0], file_system), f"file {filepath[0]} does not exist"
 
     if not columns:
         parquet_file = [
@@ -110,15 +111,20 @@ class ParquetDataset:
         columns (list of str, optional), columns to load (default: ``None`` means load all columns)
     """
 
-    def __init__(self, filepath, columns=None, file_system_config=None):
+    def __init__(
+        self, filepath, columns=None, file_system_config=None, file_system=None
+    ):
         self.filepath = filepath
         self.file_system_config = file_system_config or {}
         self.file_system = None  # defaults to local
-        fs_type = self.file_system_config.get("type")
-        if fs_type:
-            if fs_type != "pyarrow_s3":
-                raise ValueError(f"Unsupported file system type: {fs_type}")
-            self.file_system = create_pyarrow_s3fs(self.file_system_config)
+        if file_system:
+            self.file_system = file_system
+        else:
+            fs_type = self.file_system_config.get("type")
+            if fs_type:
+                if fs_type != "pyarrow_s3":
+                    raise ValueError(f"Unsupported file system type: {fs_type}")
+                self.file_system = create_pyarrow_s3fs(self.file_system_config)
         self.parquet_files, self.columns = process_filepath_columns(
             self.filepath, columns, self.file_system
         )
@@ -142,14 +148,14 @@ class ParquetDataset:
         # ensure that the metadata of the group files is same
         for idx, pf in enumerate(self.parquet_files):
             assert self.num_row_groups == pf.num_row_groups, (
-                f"parquet file {self.filepaths[idx]} has {pf.num_row_groups} row groups but "
-                "{self.filepaths[0]} has {self.num_row_groups}"
+                f"parquet file {self.filepath[idx]} has {pf.num_row_groups} row groups but "
+                "{self.filepath[0]} has {self.num_row_groups}"
             )
             assert self.row_group_rows == [
                 pf.metadata.row_group(rid).num_rows for rid in self.row_groups
-            ], f"parquet file {self.filepaths[idx]} has different rows in row group than {self.filepaths[0]}"
+            ], f"parquet file {self.filepath[idx]} has different rows in row group than {self.filepath[0]}"
 
-        if fs_type in ["pyarrow_s3"]:
+        if isinstance(self.file_system, S3FileSystem):
             self.file_system = None
             self.parquet_files = []
             self.columns = columns
@@ -246,8 +252,14 @@ class ParquetConcateDataset:
         rows = []
         rows_per_group = []
         self.file_system_config = file_system_config
+        file_system = None
+        if file_system_config and file_system_config.get("type") == "pyarrow_s3":
+            file_system = create_pyarrow_s3fs(file_system_config)
         with mt.Pool(16) as p:
-            sub_ds_info = p.map(self._get_sub_ds_length, self.filepaths)
+            get_length_with_fs = functools.partial(
+                self._get_sub_ds_length, file_system=file_system
+            )
+            sub_ds_info = p.map(get_length_with_fs, self.filepaths)
         rows = [i[0] for i in sub_ds_info]
         rows_per_group = [i[1] for i in sub_ds_info]
         self.rows_per_file = rows
@@ -262,8 +274,8 @@ class ParquetConcateDataset:
         self.cur_ds_idx = 0
         self.cache = {}
 
-    def _get_sub_ds_length(self, path):
-        ds = ParquetDataset(path, file_system_config=self.file_system_config)
+    def _get_sub_ds_length(self, path, file_system=None):
+        ds = ParquetDataset(path, file_system=file_system)
         row = len(ds)
         row_groups = ds.get_sub_lengths()
         ds.close()
@@ -289,7 +301,9 @@ class ParquetConcateDataset:
         self.cur_ds_idx = cur_ds_idx
         if cur_ds_idx not in self.cache:
             cur_ds = ParquetDataset(
-                self.filepaths[cur_ds_idx], self.columns, self.file_system_config
+                self.filepaths[cur_ds_idx],
+                self.columns,
+                file_system_config=self.file_system_config,
             )
             self._clean_cache()
             self.cache[cur_ds_idx] = cur_ds
